"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connection = void 0;
const querystring_1 = __importDefault(require("querystring"));
const axios_1 = __importDefault(require("axios"));
const authUrl = 'https://sso.ci.ford.com/';
const vehiclesUrl = 'https://services.cx.ford.com/api/dashboard/v1/users/vehicles';
const defaultAppId = '71A3AD0A-CF46-4CCF-B473-FC7FE5BC4592';
const clientId = '9fb503e0-715b-47e8-adfd-ad4b7770f73b';
const userAgent = 'FordPass/5 CFNetwork/1333.0.4 Darwin/21.5.0';
const headers = {
    'User-Agent': userAgent,
    'Accept-Language': 'en-US,en;q=0.9',
    'Accept-Encoding': 'gzip, deflate, br',
};
class Connection {
    constructor(config, log) {
        var _a;
        this.config = config;
        this.log = log;
        this.applicationId = ((_a = config.options) === null || _a === void 0 ? void 0 : _a.region) || defaultAppId;
    }
    auth() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = authUrl + 'oidc/endpoint/default/token';
            const options = {
                method: 'POST',
                url: url,
                headers: Object.assign({ 'Content-Type': 'application/x-www-form-urlencoded' }, headers),
                data: querystring_1.default.stringify({
                    client_id: clientId,
                    grant_type: 'password',
                    username: this.config.username,
                    password: this.config.password,
                }),
            };
            try {
                const result = yield (0, axios_1.default)(options);
                if (result.status === 200 && result.data.access_token) {
                    const nextResult = yield axios_1.default.post('https://api.mps.ford.com/api/token/v2/cat-with-ci-access-token', {
                        ciToken: result.data.access_token,
                    }, {
                        headers: Object.assign({ 'Content-Type': 'application/json', 'Application-Id': this.applicationId }, headers),
                    });
                    if (nextResult.status === 200 && nextResult.data.access_token) {
                        this.config.access_token = nextResult.data.access_token;
                        this.config.refresh_token = nextResult.data.refresh_token;
                        return nextResult.data;
                    }
                    else {
                        this.log.error(`Auth failed with status: ${nextResult.status}`);
                    }
                }
                else {
                    this.log.error(`Auth failed with status: ${result.status}`);
                }
                return;
            }
            catch (error) {
                this.log.error(`Auth failed with error: ${error.code || error.response.status}`);
                return;
            }
        });
    }
    refreshAuth() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this.config.refresh_token) {
                    const result = yield axios_1.default.post('https://api.mps.ford.com/api/token/v2/cat-with-refresh-token', {
                        refresh_token: this.config.refresh_token,
                    }, {
                        headers: Object.assign({ 'Content-Type': 'application/json', 'Application-Id': this.applicationId }, headers),
                    });
                    if (result.status === 200 && result.data.access_token) {
                        this.config.access_token = result.data.access_token;
                        this.config.refresh_token = result.data.refresh_token;
                        return result.data;
                    }
                    else {
                        this.log.error(`Auth failed with status: ${result.status}`);
                    }
                }
                else {
                    return yield this.auth();
                }
                return;
            }
            catch (error) {
                this.log.error(`Auth failed with error: ${error.code || error.response.status}`);
                return;
            }
        });
    }
    getVehicles() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.config.access_token) {
                return [];
            }
            const options = {
                method: 'GET',
                url: vehiclesUrl,
                headers: Object.assign({ 'Content-Type': 'application/json', 'Auth-Token': this.config.access_token, 'Application-Id': this.applicationId }, headers),
            };
            try {
                const result = yield (0, axios_1.default)(options);
                if (result.status === 200 && result.data) {
                    return result.data;
                }
                else {
                    this.log.error(`Vehicles failed with status: ${result.status}`);
                }
                return [];
            }
            catch (error) {
                this.log.error(`Vehicles failed with error: ${error.code || error.response.status}`);
                return [];
            }
        });
    }
}
exports.Connection = Connection;
//# sourceMappingURL=fordpass-connection.js.map